//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * アニメーション圧縮ヘッダ
 * @author Junpee
 */

#ifndef ANIMATION_COMPRESSOR_H_
#define ANIMATION_COMPRESSOR_H_

namespace Lamp{

class VectorInterpolationCompressor;
class VectorInterpolator;
class RotationInterpolationCompressor;
class RotationInterpolator;
class Animation;
class AnimationSet;
class CameraAnimation;
class SceneNodeAnimation;
class CharacterModelAnimation;

//------------------------------------------------------------------------------
/**
 * アニメーション圧縮 
 */
class AnimationCompressor{
public:
	/**
	 * コンストラクタ
	 */
	AnimationCompressor();

	/**
	 * デストラクタ
	 */
	virtual ~AnimationCompressor();

	//--------------------------------------------------------------------------
	// 圧縮
	//--------------------------------------------------------------------------
	/**
	 * 圧縮
	 * @param animation 圧縮するアニメーション
	 */
	virtual void compress(Animation* animation);

	//--------------------------------------------------------------------------
	// パラメータ
	//--------------------------------------------------------------------------
	/**
	 * スケール誤差の設定
	 * @param scaleTolerance スケール誤差
	 */
	virtual void setScaleTolerance(float scaleTolerance){
		Assert(scaleTolerance >= 0.f)
		scaleTolerance_ = scaleTolerance;
	}

	/**
	 * スケール誤差の取得
	 * @return スケール誤差
	 */
	virtual float getScaleTolerance() const{ return scaleTolerance_; }

	//--------------------------------------------------------------------------
	/**
	 * 回転誤差の設定
	 * @param rotationTolerance 回転誤差
	 */
	virtual void setRotationTolerance(float rotationTolerance){
		Assert(rotationTolerance >= 0.f)
		rotationTolerance_ = rotationTolerance;
	}

	/**
	 * 回転誤差の取得
	 * @return 回転誤差
	 */
	virtual float getRotationTolerance() const{
		return rotationTolerance_;
	}

	//--------------------------------------------------------------------------
	/**
	 * 移動誤差の設定
	 * @param translationTolerance 移動誤差
	 */
	virtual void setTranslationTolerance(float translationTolerance){
		Assert(translationTolerance >= 0.f)
		translationTolerance_ = translationTolerance;
	}

	/**
	 * 移動誤差の取得
	 * @return 移動誤差
	 */
	virtual float getTranslationTolerance() const{
		return translationTolerance_;
	}

protected:
	//--------------------------------------------------------------------------
	// 圧縮
	//--------------------------------------------------------------------------
	/**
	 * アニメーションの圧縮
	 * @param animation アニメーション
	 */
	virtual void compressAnimation(Animation* animation);

	/**
	 * アニメーションセットの圧縮
	 * @param animation アニメーションセット
	 */
	virtual void compressAnimationSet(AnimationSet* animation);

	/**
	 * カメラアニメーションの圧縮
	 * @param animation カメラアニメーション
	 */
	virtual void compressCameraAnimation(CameraAnimation* animation);

	/**
	 * シーンノードアニメーションの圧縮
	 * @param animation シーンノードアニメーション
	 */
	virtual void compressSceneNodeAnimation(SceneNodeAnimation* animation);

	/**
	 * キャラクタモデルアニメーションの圧縮
	 * @param animation キャラクタモデルアニメーション
	 */
	virtual void compressCharacterModelAnimation(
		CharacterModelAnimation* animation);

	//--------------------------------------------------------------------------
	// ユーティリティ
	//--------------------------------------------------------------------------
	/**
	 * スケールの圧縮
	 * @param interpolator スケール補間
	 */
	virtual VectorInterpolator* compressScale(VectorInterpolator* interpolator);

	/**
	 * 回転の圧縮
	 * @param interpolator 回転補間
	 */
	virtual RotationInterpolator* compressRotation(
		RotationInterpolator* interpolator);

	/**
	 * 移動の圧縮
	 * @param interpolator 移動補間
	 */
	virtual VectorInterpolator* compressTranslation(
		VectorInterpolator* interpolator);

private:
	//--------------------------------------------------------------------------
	// コピーコンストラクタの隠蔽
	AnimationCompressor(const AnimationCompressor& copy);

	// 代入コピーの隠蔽
	void operator =(const AnimationCompressor& copy);

	// スケール誤差
	float scaleTolerance_;
	// 回転誤差
	float rotationTolerance_;
	// 移動誤差
	float translationTolerance_;
	// スケール圧縮
	VectorInterpolationCompressor* scaleCompressor_;
	// 回転圧縮
	RotationInterpolationCompressor* rotationCompressor_;
	// 移動圧縮
	VectorInterpolationCompressor* translationCompressor_;
};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of ANIMATION_COMPRESSOR_H_
//------------------------------------------------------------------------------
